
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>

#include "COM.h"


#define TIME_POS		7
#define VALID_POS		14
#define LAT_POS			16
#define LAT_SIGN_POS    26
#define LONG_POS		28
#define LONG_SIGN_POS	39

#define TIMEOUT			2

struct POSITION
{
 unsigned char lat_g;
 unsigned char lat_p;
 unsigned char lat_s;
 char lat_sign;
 unsigned char long_g;
 unsigned char long_p;
 unsigned char long_s;
 char long_sign;
 unsigned char ora;
 unsigned char minuti;
 unsigned char secondi; 
};


struct POSITION extract_data(char nmea_string[])
{
 struct POSITION pos;
 char substring[16];
 float primi, secondi;

 // orario
 strncpy(substring,&nmea_string[TIME_POS],2);
 substring[2] = '\0';
 pos.ora = atoi(substring);
 strncpy(substring,&nmea_string[TIME_POS+2],2);
 substring[2] = '\0';
 pos.minuti = atoi(substring);
 strncpy(substring,&nmea_string[TIME_POS+4],2);
 substring[2] = '\0';
 pos.secondi = atoi(substring);
 
 // latitudine
 strncpy(substring,&nmea_string[LAT_POS],2);
 substring[2] = '\0';
 pos.lat_g = atoi(substring);
 strncpy(substring,&nmea_string[LAT_POS+2],7);
 substring[7] = '\0';
 primi = atof(substring);
 pos.lat_p = (int)floor(primi);
 secondi = (primi - floor(primi))*60;
 pos.lat_s = (int)floor(secondi+0.5);
 pos.lat_sign = nmea_string[LAT_SIGN_POS];

 // longitudine
 strncpy(substring,&nmea_string[LONG_POS],3);
 substring[3] = '\0';
 pos.long_g = atoi(substring);
 strncpy(substring,&nmea_string[LONG_POS+3],7);
 substring[7] = '\0';
 primi = atof(substring);
 pos.long_p = (int)floor(primi);
 secondi = (primi - floor(primi))*60;
 pos.long_s = (int)floor(secondi+0.5);
 pos.long_sign = nmea_string[LONG_SIGN_POS];

 return pos;
}




void main(void)
{
 HANDLE com;
 int i, n, p;
 char buffer[64], string[256];
 struct POSITION pos;
 time_t last;
 
 // apertura porta seriale
 if ((long)(com = COM_open(0, READ, 4800, 'N', 8, 1, NO_HANDSHAKE)) > 0)
   {
    p = 0;
	last=time(NULL);
    while ((time(NULL)-last) < TIMEOUT)
         {
          n = COM_read(com, buffer, sizeof(buffer)); // ricezione
          if (n > 0)
            {
             for (i=0; i<n; i++) // ricerca caratteri iniziale/finale
			    {
				 if (buffer[i] == '$') // carattere iniziale
				   {
					string[0] = buffer[i];
					p = 1;
				   }
				 else
				     {
					  if (buffer[i] == '\n') // carattere finale
					    {
						 string[p] = '\0'; // fine stringa
						 if (strncmp(string,"$GPRMC",6)==0)
						   {
							last=time(NULL);
							if (string[VALID_POS] == 'A') // valid GPS data
							  {
							   // estrazione dati da stringa "GPRMC"
							   pos = extract_data(string);
							   // visualizzazione posizione e orario
							   printf("%02i:%02i:%02i %i^%02i'%02i\"%c %i^%02i'%02i\"%c\r\n",pos.ora,pos.minuti,pos.secondi,pos.lat_g,pos.lat_p,pos.lat_s,pos.lat_sign,pos.long_g,pos.long_p,pos.long_s,pos.long_sign);
							  }
						   }
						 p = 0;
					    }
					  else // carattere intermedio
					      {
						   string[p] = buffer[i]; // copia carattere
						   p++;
					      }
				     }

				} // fine ciclo for
            }
         } // fine ciclo while
   }
 else
     printf("Errore apertura porta seriale\r\n");
 COM_close(com);
}
